package ufida.fasterxml.jackson.databind.ser.extend;

import java.io.IOException;
import java.lang.annotation.Annotation;
import java.util.List;

import ufida.fasterxml.jackson.annotation.JsonProperty;
import ufida.fasterxml.jackson.core.JsonGenerationException;
import ufida.fasterxml.jackson.core.JsonGenerator;
import ufida.fasterxml.jackson.core.JsonProcessingException;
import ufida.fasterxml.jackson.databind.JsonSerializer;
import ufida.fasterxml.jackson.databind.SerializerProvider;

public class CustomArraySerializer extends JsonSerializer<List> {
	
	protected JsonSerializer<List> _serializer;
	
	@Override
	public void serialize(List value, JsonGenerator jgen, SerializerProvider provider) throws IOException, JsonProcessingException {
		Annotation an = (value.getClass()).getAnnotation(JsonProperty.class);
		if(1 == value.size()) {
			//jgen.writeStartArrayss();
	        if (_serializer == null) {
	            serializeContents(value, jgen, provider);
	        } else {
	            serializeUsingCustom(value, jgen, provider);
	        }
	        //jgen.writeEndArrayss();
		}else {
			jgen.writeStartArray();
	        if (_serializer == null) {
	            serializeContents(value, jgen, provider);
	        } else {
	            serializeUsingCustom(value, jgen, provider);
	        }
	        jgen.writeEndArray();
		}
	}

	private final void serializeContents(List value, JsonGenerator jgen, SerializerProvider provider)
	        throws IOException, JsonGenerationException
	    {
	        int i = 0;
	        try {
	            final int len = value.size();
	            for (; i < len; ++i) {
	                Object str = value.get(i);
	                if (str == null) {
	                    provider.defaultSerializeNull(jgen);
	                } else {
	                    jgen.writeObject(str);
	                }
	            }
	        } catch (Exception e) {
	           // wrapAndThrow(provider, e, value, i);
	        }
	    }

	    private final void serializeUsingCustom(List value, JsonGenerator jgen, SerializerProvider provider)
	        throws IOException, JsonGenerationException
	    {
	        int i = 0;
	        try {
	            final int len = value.size();
	            final JsonSerializer<List> ser = _serializer;
	            for (i = 0; i < len; ++i) {
	                Object str = value.get(i);
	                if (str == null) {
	                    provider.defaultSerializeNull(jgen);
	                } else {
	                    ser.serialize((List) str, jgen, provider);
	                }
	            }
	        } catch (Exception e) {
	            //wrapAndThrow(provider, e, value, i);
	        }
	    }
	
}
